using System;
using System.Collections.Generic;
using System.Linq;

namespace CustomQueryOperators {
	public static class LargeSumOperator {
		public static long LargeSum(this IEnumerable<int> list) {
			if (list == null)
				throw new ArgumentNullException("list", "list is null.");

			long sum = 0;

			checked {
				foreach (int item in list) {
					sum += item;
				}
			}

			return sum;
		}

		public static long? LargeSum(this IEnumerable<int?> list) {
			if (list == null)
				throw new ArgumentNullException("list", "list is null.");

			long? sum = 0;

			checked {
				foreach (int? item in list) {
					if (item.HasValue)
						sum += item;
				}
			}

			return sum;
		}

		public static long LargeSum<T>(this IEnumerable<T> list, Func<T, int> selector) {
			return LargeSum(Enumerable.Select(list, selector));
		}

		public static long? LargeSum<T>(this IEnumerable<T> list, Func<T, int?> selector) {
			return LargeSum(Enumerable.Select(list, selector));
		}
	}
}
