using System;
using System.Collections.Generic;
using System.Linq;
using QueryEngine;
using System.Linq.Expressions;
using System.Collections;
using System.Reflection;
using System.Text.RegularExpressions;

namespace LINQFrontend {
	public class LegacyEngineLinqAdapter : IQueryable<LegacyEngineResult>, IQueryProvider {
		public LegacyEngineLinqAdapter(LegacyEngine legacyEngine) {
			this.legacyEngine = legacyEngine;
		}

		LegacyEngine legacyEngine;
		Expression expression;

		#region Important methods
		#region IQueryProvider.CreateQuery<TResult>
		IQueryable<TResult> IQueryProvider.CreateQuery<TResult>(Expression expression) {
			this.expression = expression;
			return (IQueryable<TResult>) this;
		}
		#endregion

		#region IQueryable.Expression
		Expression IQueryable.Expression {
			get {
				return expression ?? Expression.Constant(this);
			}
		}
		#endregion

		#region IEnumerable<LegacyEngineResult>.GetEnumerator

		IEnumerator<LegacyEngineResult> IEnumerable<LegacyEngineResult>.GetEnumerator( ) {
			var key = ParseExpression(expression);
			var resultstring = legacyEngine.Query("GET ME " + key);

			var result = ConvertResultString(resultstring);
			result.Key = key;

			return ((IEnumerable<LegacyEngineResult>) (new[] { result })).GetEnumerator( );
		}

		#endregion

		#region IQueryable.Provider

		IQueryProvider IQueryable.Provider {
			get {
				return this;
			}
		}

		#endregion
		#endregion

		// My own method, to convert result strings
		LegacyEngineResult ConvertResultString(string resultstring) {
			// Here we are free to look at the format of the string and decide whether
			// it's supported or not, then parse the contents into the object
			// as needed.

			Match match = Regex.Match(resultstring, @"^\s*(?<text>.*?)\s*,\s*(?<num1>\d+)\s*,\s*(?<num2>\d+)\s*$");
			if (match.Success) {
				return new LegacyEngineResult
				{
					StringValue = match.Groups["text"].Value,
					IntVal1 = Int32.Parse(match.Groups["num1"].Value),
					IntVal2 = Int32.Parse(match.Groups["num2"].Value)
				};
			}

			match = Regex.Match(resultstring, @"^\s*(?<num1>\d+)\s*,\s*(?<text>.*?)\s*$");
			if (match.Success) {
				return new LegacyEngineResult
				{
					StringValue = match.Groups["text"].Value,
					IntVal1 = Int32.Parse(match.Groups["num1"].Value),
				};
			}

			return new LegacyEngineResult
			{
				StringValue = resultstring
			};
		}

		// My own method, to interpret the expression
		#region ParseExpression
		string ParseExpression(Expression expression) {
			//ExpressionDumper.Output(expression);
			//Console.ReadLine( );

			if (expression.NodeType != ExpressionType.Call)
				throw new NotSupportedException("Only MethodCall expressions supported.");

			var whereMethodCall = (MethodCallExpression) expression;
			if (whereMethodCall.Method.Name != "Where")
				throw new NotSupportedException("Only Where MethodCalls supported.");

			if (whereMethodCall.Arguments.Count != 2)
				throw new NotSupportedException("Only argument count 2 supported.");

			if (((UnaryExpression) whereMethodCall.Arguments[1]).Operand.NodeType != ExpressionType.Lambda)
				throw new NotSupportedException("Only Where argument type Lambda supported.");

			var whereLambda = ((LambdaExpression) ((UnaryExpression) whereMethodCall.Arguments[1]).Operand);

			if (whereLambda.Body.NodeType != ExpressionType.Equal)
				throw new NotSupportedException("Only Equal expressions supported for lambda body.");

			var equalExpression = (BinaryExpression) whereLambda.Body;

			if (equalExpression.Left.NodeType != ExpressionType.MemberAccess)
				throw new NotSupportedException("Only member access supported for left hand side.");

			var keyAccess = (MemberExpression) equalExpression.Left;
			if (keyAccess.Member != typeof(LegacyEngineResult).GetProperty("Key"))
				throw new NotSupportedException("Only references to Key supported.");

			if (equalExpression.Right.NodeType != ExpressionType.Constant)
				throw new NotSupportedException("Only constant strings supported for right hand side.");

			var valueConst = (ConstantExpression) equalExpression.Right;
			return (string) valueConst.Value;
		}
		#endregion

		#region Other stuff

		#region IQueryable.ElementType
		Type IQueryable.ElementType {
			get {
				throw new Exception("The method or operation is not implemented.");
			}
		}
		#endregion

		#region IEnumerable.GetEnumerator

		IEnumerator IEnumerable.GetEnumerator( ) {
			// Parse the expression, execute the query and return
			// the list of results

			throw new Exception("The method or operation is not implemented.");
		}

		#endregion

		#region IQueryProvider Members

		IQueryable IQueryProvider.CreateQuery(Expression expression) {
			throw new NotImplementedException( );
		}

		TResult IQueryProvider.Execute<TResult>(Expression expression) {
			throw new NotImplementedException( );
		}

		object IQueryProvider.Execute(Expression expression) {
			throw new NotImplementedException( );
		}

		#endregion

		#endregion


	}
}
