using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using QueryApplication.Complete.DataWebService;
using System.Linq.Expressions;

namespace QueryApplication.Complete {
	public class DataWebServiceQuery : IQueryable<CountryInfo>, IQueryProvider {
		public DataWebServiceQuery(Service dataWebService) {
			this.dataWebService = dataWebService;
		}

		Service dataWebService;
		Expression expression;

		#region Important methods
		#region IQueryable.Provider

		IQueryProvider IQueryable.Provider {
			get {
				return this;
			}
		}

		#endregion
		#region IQueryable.Expression
		Expression IQueryable.Expression {
			get {
				return expression ?? Expression.Constant(this);
			}
		}
		#endregion
		#region IQueryProvider.CreateQuery<TResult>
		IQueryable<TResult> IQueryProvider.CreateQuery<TResult>(Expression expression) {
			this.expression = expression;
			return (IQueryable<TResult>) this;
		}
		#endregion
		#region IEnumerable<CountryInfo>.GetEnumerator

		IEnumerator<CountryInfo> IEnumerable<CountryInfo>.GetEnumerator( ) {
			string startsWithString = ParseExpression(expression);
			CountryInfo[] result = dataWebService.GetFilteredData(startsWithString);
			return ((IEnumerable<CountryInfo>) result).GetEnumerator( );
		}

		#endregion


		#endregion

		// My own method, to interpret the expression
		#region ParseExpression
		string ParseExpression(Expression expression) {
			//ExpressionDumper.Output(expression);
			//Console.ReadLine( );

			if (expression.NodeType != ExpressionType.Call)
				throw new NotSupportedException("Only Call expressions supported.");

			MethodCallExpression methodCall = (MethodCallExpression) expression;
			if (methodCall.Method.Name != "Where")
				throw new NotSupportedException("Only Where calls supported.");

			if (methodCall.Arguments.Count != 2)
				throw new NotSupportedException("Only argument count 2 supported.");

			if (methodCall.Arguments[1].NodeType != ExpressionType.Quote)
				throw new NotSupportedException("Only Quote supported here.");

			UnaryExpression quote = ((UnaryExpression) methodCall.Arguments[1]);

			if (quote.Operand.NodeType != ExpressionType.Lambda)
				throw new NotSupportedException("Only Where argument type Lambda supported.");

			LambdaExpression lambda = ((LambdaExpression) quote.Operand);

			if (lambda.Body.NodeType != ExpressionType.Call)
				throw new NotSupportedException("Only Call expressions supported for lambda body.");

			MethodCallExpression lambdaMethodCall = (MethodCallExpression) lambda.Body;
			if (lambdaMethodCall.Method.Name != "StartsWith")
				throw new NotSupportedException("Only lambda body calls to string.StartsWith supported.");

			ConstantExpression constantExpression = (ConstantExpression) lambdaMethodCall.Arguments[0];
			if (constantExpression.Type != typeof(string))
				throw new NotSupportedException("Only string arguments to string.StartsWith supported.");

			return (string) constantExpression.Value;
		}
		#endregion

		#region Other stuff
		#region IQueryable.ElementType
		Type IQueryable.ElementType {
			get {
				throw new Exception("The method or operation is not implemented.");
			}
		}
		#endregion
		
		#region IEnumerable.GetEnumerator

		IEnumerator IEnumerable.GetEnumerator( ) {
			throw new Exception("The method or operation is not implemented.");
		}

		#endregion
		#region IQueryProvider Members


		IQueryable IQueryProvider.CreateQuery(Expression expression) {
			throw new NotImplementedException( );
		}

		TResult IQueryProvider.Execute<TResult>(Expression expression) {
			throw new NotImplementedException( );
		}

		object IQueryProvider.Execute(Expression expression) {
			throw new NotImplementedException( );
		}

		#endregion
		#endregion

	}
}
